'use client'

import Image from 'next/image'
import { useEffect, useState } from 'react'

import { zodResolver } from '@hookform/resolvers/zod'
import Cookies from 'js-cookie'
import { Gift } from 'lucide-react'
import { useForm } from 'react-hook-form'
import * as z from 'zod'

import { Button } from '@/components/ui/button'
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form'
import { Input } from '@/components/ui/input'
import { Label } from '@/components/ui/label'
import { Textarea } from '@/components/ui/textarea'
import { useToast } from '@/components/ui/use-toast'
import { CREDIT_CARD_ID, IDRAM_ID } from '@/constants'
import useGetAccountData from '@/hooks/server/useGetAccountData'
import { cn, convertToMoney, formatInputValue, handleIdram } from '@/lib/utils'
import axiosClient from '@/services/axiosClient'

const formSchema = z.object({
  recipient_name: z.string().min(1, { message: 'Ստացողի Անունը պարտադիր է' }),
  recipient_email: z
    .string()
    .min(1, { message: 'ստացողի Էլ․ հասցեն պարտադիր է' })
    .email({
      message: 'Խնդրում ենք մուտքագրել վավեր էլ․ հասցե',
    }),
  sender_name: z.string().min(1, { message: 'Նվիրողի անունը պարտադիր է' }),
  sender_email: z
    .string()
    .min(1, { message: 'ստացողի Էլ․ հասցեն պարտադիր է' })
    .email({
      message: 'Խնդրում ենք մուտքագրել վավեր էլ․ հասցե',
    }),
  amount: z
    .string()
    .transform((value) => Number(formatInputValue(value, 'integerWithDecimal')))
    .transform((num) => (isNaN(num) ? 0 : num))
    .refine((num) => num >= 5000 && num <= 100000, {
      message: 'Արժեքը պետք է լինի 5,000 ֏ և 100,000 ֏ միջակայքում',
    })
    .transform((num) => num.toString()),
  message: z.string(),
})

const OnlineGiftCard = () => {
  const [token, setToken] = useState<string | undefined>(undefined)
  const [paymentType, setPaymentType] = useState<
    typeof CREDIT_CARD_ID | typeof IDRAM_ID
  >(CREDIT_CARD_ID)
  const [giftCardCode, setGiftCardCode] = useState('')
  const [isGiftCardLoading, setIsGiftCardLoading] = useState(false)
  const { toast } = useToast()
  const [giftCardAmount, setGiftCardAmount] = useState(0)
  const { accountData } = useGetAccountData()

  const form = useForm({
    resolver: zodResolver(formSchema),
    defaultValues: {
      recipient_name: '',
      recipient_email: '',
      sender_name:
        accountData?.name && accountData?.lastname
          ? `${accountData.name} ${accountData.lastname}`
          : accountData?.name || '',
      sender_email: accountData?.email || '',
      amount: '',
      message: '',
    },
  })

  useEffect(() => {
    setToken(Cookies.get('siamoods_token'))
  }, [])

  async function onSubmit(values: z.infer<typeof formSchema>) {
    const postValues = {
      ...values,
      payment_method: paymentType,
      sender_name: values.sender_name,
      sender_email: values.sender_email,
    }
    try {
      const res = await axiosClient.post('api/gift-cards', postValues)
      const data = res.data
      if (paymentType === IDRAM_ID) {
        handleIdram({
          EDP_REC_ACCOUNT: data?.payment_info?.account,
          EDP_DESCRIPTION: data?.payment_info?.description,
          EDP_AMOUNT: data?.payment_info?.amount,
          EDP_BILL_NO: data?.payment_key,
        })
        localStorage.setItem('payment_method', IDRAM_ID)
        localStorage.setItem('totalPrice', data?.payment_info?.amount)
        localStorage.setItem('isGiftCardPurchase', 'true')
        return
      }
    } catch (error) {
      console.log(error)
    }
  }

  const handleCheckGiftCard = async () => {
    if (!giftCardCode) return
    setIsGiftCardLoading(true)
    try {
      const res = await axiosClient.get(
        `api/gift-cards/check?gift_card=${giftCardCode}`
      )
      setGiftCardAmount(res.data.amount)
    } catch (error) {
      toast({
        title: 'Նվեր քարտը գտնված չէ',
        variant: 'destructive',
      })
    } finally {
      setIsGiftCardLoading(false)
    }
  }

  return (
    <div className='flex flex-col items-center p-12 pt-0 sm:p-2'>
      <div className='flex w-full max-w-7xl flex-col items-center gap-8 rounded-2xl p-8 shadow-2xl sm:p-4'>
        <div className='relative isolate flex w-full max-w-full justify-center'>
          <h1 className='bg-background px-8 text-2xl text-primary sm:text-xl'>
            Օնլայն Նվեր Քարտ
          </h1>
          <div className='absolute top-1/2 -z-10 h-px w-full  translate-y-1/2 bg-primary' />
        </div>
        <div className='flex w-full max-w-7xl gap-8 lg:flex-col lg:gap-12'>
          <Form {...form}>
            <form
              onSubmit={form.handleSubmit(onSubmit)}
              className='flex w-full flex-[2] flex-col justify-center gap-6'
            >
              <h2 className='whitespace-nowrap text-lg'>Նվիրել քարտ</h2>

              <div className='grid grid-cols-2 gap-4 sm:grid-cols-1'>
                <FormField
                  name='sender_name'
                  render={({ field }) => (
                    <FormItem className='order-1'>
                      <FormLabel required={true}>
                        Նվիրողի անուն ազգանուն
                      </FormLabel>
                      <FormControl>
                        <Input {...field} disabled={token ? true : false} />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <FormField
                  name='recipient_name'
                  render={({ field }) => (
                    <FormItem className='sm:or order-2'>
                      <FormLabel required={true}>
                        Ստացողի անուն ազգանուն
                      </FormLabel>
                      <FormControl>
                        <Input {...field} />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <FormField
                  name='sender_email'
                  render={({ field }) => (
                    <FormItem className='order-3 sm:order-2'>
                      <FormLabel required={true}>Նվիրողի էլ․ հասցե</FormLabel>
                      <FormControl>
                        <Input
                          {...field}
                          type='email'
                          disabled={token ? true : false}
                        />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <FormField
                  name='recipient_email'
                  render={({ field }) => (
                    <FormItem className='order-4'>
                      <FormLabel required={true}>Ստացողի էլ․ հասցե</FormLabel>
                      <FormControl>
                        <Input {...field} type='email' />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
              </div>
              <FormField
                name='amount'
                render={({ field }) => {
                  return (
                    <FormItem className='mb-3'>
                      <FormLabel required={true}>Գումար</FormLabel>
                      <FormControl>
                        <Input {...field} inputType='integer' maxLength={6} />
                      </FormControl>
                      <FormMessage className='-bottom-6 text-sm text-muted-foreground sm:static sm:whitespace-normal'>
                        Արժեքը պետք է լինի 5,000 ֏ և 100,000 ֏ միջակայքում
                      </FormMessage>
                    </FormItem>
                  )
                }}
              />
              <FormField
                name='message'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel>Ձեր մաղթանքը</FormLabel>
                    <FormControl>
                      <Textarea {...field} />
                    </FormControl>
                  </FormItem>
                )}
              />
              <div className='flex gap-4 sm:flex-col'>
                <div
                  className={cn(
                    'relative flex min-h-[7.5rem] flex-1 cursor-pointer flex-col items-center justify-center rounded-lg border p-4 pt-3 ring-primary transition-shadow hover:ring-2',
                    paymentType === CREDIT_CARD_ID && 'ring-2'
                  )}
                  onClick={() => setPaymentType(CREDIT_CARD_ID)}
                >
                  <Image
                    src='/icons/amex-mastercard-visa-logo.png'
                    alt='credit card'
                    width={180}
                    height={80}
                    className='pb-3'
                  />
                  <div className='absolute bottom-4 left-1/2 -translate-x-1/2 whitespace-nowrap'>
                    Կրեդիտ քարտ
                  </div>
                </div>
                <div
                  className={cn(
                    'relative flex min-h-[7.5rem] flex-1 cursor-pointer flex-col items-center justify-center rounded-lg border p-4 pt-3 ring-primary transition-shadow hover:ring-2',
                    paymentType === IDRAM_ID && 'ring-2'
                  )}
                  onClick={() => setPaymentType(IDRAM_ID)}
                >
                  <Image
                    src='/icons/idram.png'
                    alt='idram'
                    width={120}
                    height={40}
                    className='pb-3'
                  />
                  <div className='absolute bottom-4 left-1/2 -translate-x-1/2 whitespace-nowrap'>
                    Idram
                  </div>
                </div>
              </div>
              {/* <Link href='/checkout'> */}
              <Button className='mt-4 w-full'>Անցնել Վճարմանը</Button>
              {/* </Link> */}
            </form>
          </Form>
          <div className='relative flex flex-1 flex-col gap-4 overflow-hidden rounded-lg p-8'>
            <div className='gradient' />
            <h2 className='whitespace-nowrap text-lg '>Ստուգել Նվեր քարտը</h2>
            <div className='space-y-2'>
              <Label htmlFor='gift-card-code'>Նվեր քարտի կոդ</Label>
              <div className='relative'>
                <Input
                  id='gift-card-code'
                  value={giftCardCode}
                  onChange={(e) => setGiftCardCode(e.target.value)}
                  onKeyDown={(e) => {
                    if (e.key === 'Enter' && !isGiftCardLoading) {
                      handleCheckGiftCard()
                    }
                  }}
                  maxLength={16}
                  className='pl-9'
                />
                <Gift className='pointer-events-none absolute left-2 top-1.5 text-muted-foreground' />
              </div>
            </div>

            <Button
              onClick={handleCheckGiftCard}
              variant='outlineSecondary'
              disabled={isGiftCardLoading}
            >
              Ստուգել
            </Button>

            {!!giftCardAmount && (
              <div className='space-y-2'>
                <Label>Նվեր քարտի գումարը</Label>
                <div className='text-2xl font-bold text-primary'>
                  {convertToMoney(giftCardAmount)}
                </div>
              </div>
            )}
          </div>
        </div>
      </div>
    </div>
  )
}

export default OnlineGiftCard
