import { Metadata } from 'next'
import { notFound } from 'next/navigation'
import { Suspense } from 'react'

import {
  HydrationBoundary,
  QueryClient,
  dehydrate,
} from '@tanstack/react-query'

import Store from '@/app/store/Store'
import axiosServer from '@/services/axiosServer'
import { ProductType } from '@/types'
import { JewelryTypes } from '@/types/store.type'

export const dynamic = 'force-dynamic'

export async function generateMetadata(): Promise<Metadata> {
  const res = await axiosServer('api/store-data')
  const product: JewelryTypes = res.data

  if (!product) return notFound()

  return {
    title: product.seo_meta.meta_title,
    description: product.seo_meta.meta_description,
    keywords: product.seo_meta.meta_keywords,
    category: 'products',
    openGraph: {
      title: product.seo_meta.meta_title,
      description: product.seo_meta.meta_description,
      type: 'website',
      siteName: 'SiaMoods',
      url: `${process.env.NEXT_PUBLIC_APP_URL}store`,
    },
    alternates: {
      canonical: `${process.env.NEXT_PUBLIC_APP_URL}store`,
    },
  }
}

export default async function page({
  searchParams = {},
}: {
  searchParams: any
}) {
  const queryClient = new QueryClient()
  await queryClient.prefetchQuery({
    queryKey: ['store'],
    queryFn: async (): Promise<JewelryTypes | undefined> => {
      const res = await axiosServer('api/store-data')
      return res.data
    },
  })

  await queryClient.prefetchQuery({
    queryKey: [
      'products',
      {
        page: searchParams.page || '1',
        ...searchParams,
      },
    ],
    queryFn: async (): Promise<
      | {
          data: ProductType[]
          minValue: number
          maxValue: number
          meta: {
            current_page: number
            last_page: number
          }
        }
      | undefined
    > => {
      const res = await axiosServer('api/products', {
        params: {
          page: searchParams.page || '1',
          stone: searchParams.stone?.split(','),
          type: searchParams.type?.split(','),
          filter: searchParams.filter?.split(','),
          sub_type: searchParams.sub_type?.split(','),
          balance: searchParams.balance,
          discount: searchParams.discount,
          sort_by: searchParams.sort_by,
          search: searchParams.search,
          min_price: searchParams.min_price,
          max_price: searchParams.max_price,
        },
      })
      return res.data
    },
  })

  return (
    <div>
      <HydrationBoundary state={dehydrate(queryClient)}>
        <Suspense fallback={<div>Loading...</div>}>
          <Store />
        </Suspense>
      </HydrationBoundary>
    </div>
  )
}
