'use client'

import { useMutation, useQueryClient } from '@tanstack/react-query'
import { AxiosError } from 'axios'
import Cookies from 'js-cookie'
import { Heart } from 'lucide-react'
import { useRouter } from 'next-nprogress-bar'

import AddedToCartDialog from '@/components/dialogs/AddedToCartDialog'
import { Button } from '@/components/ui/button'
import { InputQuantity } from '@/components/ui/inputQuantity'
import axiosClient from '@/services/axiosClient'
import { useUserStore } from '@/store/user'
import type { ProductType } from '@/types'

import NotifyIfAvailableDialog from './dialogs/NotifyIfAvailableDialog'
import { useToast } from './ui/use-toast'

const AddToCart = ({
  product,
  haysellId,
  quantity,
  setQuantity,
  onChange,
  available,
  variant,
  isFavoriteInitial = false,
}: {
  product?: ProductType
  haysellId?: number
  quantity: number | string
  setQuantity: (value: number) => void
  onChange: (e: React.ChangeEvent<HTMLInputElement>) => void
  available: number | undefined
  variant?: string
  isFavoriteInitial?: boolean
}) => {
  const token = Cookies.get('siamoods_token')
  const router = useRouter()
  const queryClient = useQueryClient()
  const { toast } = useToast()
  const { currentFavorites, addOrRemoveFavorite } = useUserStore()

  const favoriteProduct = currentFavorites?.find(
    (item) => item.haysell_id === haysellId
  )
  const isFavorite = favoriteProduct
    ? favoriteProduct.isFavorite
    : isFavoriteInitial

  const addOrRemoveToFavoriteMutation = useMutation({
    mutationFn: (id: number) => {
      if (isFavorite) {
        return axiosClient.delete(`api/remove-favorite/${id}`)
      } else {
        return axiosClient.post(`api/add-favorite`, {
          haysell_id: id,
          quantity: 1,
          variant,
        })
      }
    },
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: ['favorites'] })
      if (haysellId) {
        addOrRemoveFavorite(haysellId)
      }
      if (process.env.NODE_ENV === 'production') {
        fbq('track', 'AddToCart', {
          content_ids: [haysellId],
          content_type: 'product',
        })
      }

      // toast({
      //   title: isFavorite ? 'Ապրանքը հեռացված է' : 'Ապրանքը ավելացված է',
      // })
      // if (isFavorite) {
      //   toast({
      //     title: 'Ապրանքը հեռացված է',
      //   })
      // }
    },
    onError: (error) => {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    },
  })

  return (
    <div className='flex flex-wrap justify-center gap-4'>
      {!!available && +available > 0 ? (
        <InputQuantity
          value={quantity}
          setValue={setQuantity}
          onChange={onChange}
          available={available}
        />
      ) : (
        <NotifyIfAvailableDialog haysellId={haysellId} variationId={variant} />
      )}

      <div className='flex gap-4'>
        {!!available && +available > 0 && (
          <AddedToCartDialog
            product={product}
            haysellId={haysellId}
            isInProduct={true}
            quantity={quantity}
            variant={variant}
          />
        )}
        <Button
          variant='outline'
          className='text-red-500 hover:text-red-500'
          disabled={addOrRemoveToFavoriteMutation.isPending}
          onClick={() => {
            if (!token) {
              router.push('/login')
              return
            }
            if (haysellId) {
              addOrRemoveToFavoriteMutation.mutate(haysellId)
            }
          }}
        >
          <Heart fill={isFavorite ? '#ef4444' : 'none'} />
        </Button>
      </div>
    </div>
  )
}

export default AddToCart
